@@ -2,6 +2,7 @@ import pathlib
 import shlex
 import argparse
 from typing import List, Optional, Dict, Any, Tuple
+import sys
 
 from rich.console import Console
 from rich.prompt import Prompt
@@ -219,6 +220,7 @@ class Chatbot:
                     gitingest_summary=self.gitingest_summary,
                     gitingest_tree=self.gitingest_tree,
                     gitingest_content_str=self.gitingest_content_str,
+                    auth_headers=getattr(self, "auth_headers", {}),
                 )
 
                 if result and isinstance(result, list):
@@ -331,6 +333,7 @@ class Chatbot:
                         target_file=selected_option["target_file"],
                         description=selected_option["description"],
                         gitingest_content_str=self.gitingest_content_str,
+                        auth_headers=getattr(self, "auth_headers", {}),
                     )
                 if result and result[0]:
                     eval_script_content, metric_name, goal, reasoning = result
@@ -380,6 +383,7 @@ class Chatbot:
                 script_content=eval_script_content,
                 script_path=eval_script_path_str,
                 target_file=selected_option["target_file"],
+                auth_headers=getattr(self, "auth_headers", {}),
             )
 
         return {
@@ -400,6 +404,7 @@ class Chatbot:
                 gitingest_summary=self.gitingest_summary,
                 gitingest_tree=self.gitingest_tree,
                 gitingest_content_str=self.gitingest_content_str,
+                auth_headers=getattr(self, "auth_headers", {}),
             )
 
         if not analysis:
@@ -541,6 +546,7 @@ class Chatbot:
                         script_content=script_content,
                         script_path=script_path,
                         target_file=selected_option["target_file"],
+                        auth_headers=getattr(self, "auth_headers", {}),
                     )
 
             self.current_step = "confirmation"
@@ -682,9 +688,9 @@ class Chatbot:
 
         # Import and execute the actual optimization function
         # (Import here to avoid circular imports)
-        from .optimizer import execute_optimization as actual_execute_optimization
+        from .optimizer import execute_optimization as execute_optimization_run
 
-        success = actual_execute_optimization(
+        success = execute_optimization_run(
             source=target_file,
             eval_command=eval_config["eval_command"],
             metric=eval_config["metric_name"],
@@ -702,6 +708,9 @@ class Chatbot:
         else:
             self.console.print("\n[bold yellow]⚠️  Optimization ended early or encountered issues.[/]")
 
+        exit_code = 0 if success else 1
+        sys.exit(exit_code)
+
     def show_and_copy_command(self, command: str) -> None:
         """Show the command and copy it to clipboard."""
         import subprocess
@@ -745,10 +754,9 @@ class Chatbot:
             self.resolved_model = self.user_specified_model
         else:
             # Use same default model selection as weco run
-            from .utils import determine_default_model, read_api_keys_from_env
+            from .utils import determine_model_for_onboarding
 
-            llm_api_keys = read_api_keys_from_env()
-            self.resolved_model = determine_default_model(llm_api_keys)
+            self.resolved_model = determine_model_for_onboarding()
 
         target_file = selected_option["target_file"]
         additional_instructions = selected_option["description"]
@@ -762,6 +770,17 @@ class Chatbot:
         self.console.print("[bold cyan]Welcome to Weco![/]")
         self.console.print(f"Let's optimize your codebase in: [cyan]{self.project_path}[/]\n")
 
+        # Mandatory authentication as per PLAN.md
+        from .auth import handle_authentication
+
+        weco_api_key, auth_headers = handle_authentication(self.console)
+        if not weco_api_key:
+            self.console.print("[yellow]Authentication is required to use Weco. Exiting...[/]")
+            return
+
+        # Store auth headers for API calls
+        self.auth_headers = auth_headers
+
         options = self.analyze_codebase_and_get_optimization_options()
         if not options:
             return
